﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Windows.Forms; 
using System.Xml;

using ComponentSpace.SAML2;
using ComponentSpace.SAML2.Metadata;

namespace Metadata
{
    public partial class Metadata : Form
    {
        X509Certificate2 m_certificate;
        
        public Metadata()
        {
            InitializeComponent();

            m_certificateStatus.Text = "Invalid Certificate";
            m_certificateStatus.BackColor = Color.Red;
        }

        private void btn_importCertificate_Click(object sender, EventArgs e)
        {
            PickCertificate form = new PickCertificate();   
            if (DialogResult.OK == form.ShowDialog())
            {
                try
                {
                    m_certificate = new X509Certificate2(form.KeyFilename, form.Password, X509KeyStorageFlags.Exportable);
                    if (!m_certificate.HasPrivateKey)
                        throw new Exception("Certificate must have a private key");

                    m_certificateStatus.Text = "Valid Certificate";
                    m_certificateStatus.BackColor = Color.Green;
                }
                catch
                {
                    m_certificateStatus.Text = "Invalid Certificate";
                    m_certificateStatus.BackColor = Color.Red;
                    MessageBox.Show("Cannot import certificate. Make sure you provide a valid certificate file (which includes the private key part) and the correct password.");  
                }
            }
        }  

        private void btn_generateMetadata_Click(object sender, EventArgs e)
        {
            if (m_certificate == null)
            {
                MessageBox.Show("Cannot create metadata without a valid certificate.");
                return;
            }

            if (!Uri.IsWellFormedUriString(textbox_EntityID.Text, UriKind.Absolute))
            {
                MessageBox.Show("Cannot create metadata, EntityID must be a valid URL.");
                return;
            }

            XmlElement metadata = CreateMetaData();
            SAMLMetadataSignature.Generate(metadata, m_certificate.PrivateKey, m_certificate); 

            // save metadata to file                    
            SaveFileDialog dialog = new SaveFileDialog();
            Uri uri = new Uri(textbox_EntityID.Text);
            dialog.FileName = "Metadata(" + uri.Host + ").xml";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                string filename = dialog.FileName.ToString();
                using (XmlWriter writer = XmlWriter.Create(filename))
                    metadata.WriteTo(writer);                
            }
        }

        // Creates Metadata
        private XmlElement CreateMetaData()
        {
            EntityDescriptor entityDescriptor = new EntityDescriptor();
            entityDescriptor.EntityID = new EntityIDType(textbox_EntityID.Text);
            entityDescriptor.SPSSODescriptors.Add(CreateSPSSODescriptor());
            return entityDescriptor.ToXml();
        }
        // Creates an SP SSO descriptor
        private SPSSODescriptor CreateSPSSODescriptor()
        {
            SPSSODescriptor spSSODescriptor = new SPSSODescriptor(); 
            spSSODescriptor.ProtocolSupportEnumeration = SAML.NamespaceURIs.Protocol;
            spSSODescriptor.KeyDescriptors.Add(CreateKeyDescriptor(m_certificate));
            spSSODescriptor.NameIDFormats.Add(SAMLIdentifiers.NameIdentifierFormats.Transient);

            return spSSODescriptor;
        }
        // Creates a KeyDescriptor from the supplied X.509 certificate
        private static KeyDescriptor CreateKeyDescriptor(X509Certificate2 x509Certificate)
        {
            KeyDescriptor keyDescriptor = new KeyDescriptor();
            KeyInfo keyInfo = CreateKeyInfo(x509Certificate);
            keyDescriptor.KeyInfo = keyInfo.GetXml();
            return keyDescriptor;
        }
        // Creates a KeyInfo from the supplied X.509 certificate
        private static KeyInfo CreateKeyInfo(X509Certificate2 x509Certificate)
        {
            KeyInfoX509Data keyInfoX509Data = new KeyInfoX509Data();
            keyInfoX509Data.AddCertificate(x509Certificate);
            KeyInfo keyInfo = new KeyInfo();
            keyInfo.AddClause(keyInfoX509Data);
            return keyInfo;
        }
    }
}
