﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;

using ComponentSpace.SAML2;
using ComponentSpace.SAML2.Assertions;
using ComponentSpace.SAML2.Bindings;
using ComponentSpace.SAML2.Metadata;
using ComponentSpace.SAML2.Protocols;
using ComponentSpace.SAML2.Profiles.SSOBrowser;

namespace SAML2SPExample
{
    public partial class AssertionConsumerService : System.Web.UI.Page
    {
        string m_uid;
        protected void Page_Load(object sender, EventArgs e)
        {
            // Read Metadata (Required to verify SAMLResonse is singed with the correct certificate)
            XmlDocument IdPMetadata = new XmlDocument();
            IdPMetadata.PreserveWhitespace = true;
            using (XmlReader xmlreader = XmlReader.Create("https://hub-s.entree.kennisnet.nl/openaselect/profiles/saml2/"))
                IdPMetadata.Load(xmlreader);

            // Verify Metadata signature
            if (!SAMLMetadataSignature.Verify(IdPMetadata.DocumentElement))
                throw new Exception("Kennisnet metadata has invalid signature");

            // Get the SAMLResponse from the HTTPRequest
            String relayState = null;
            XmlElement samlResponseXml = null;
            ServiceProvider.ReceiveSAMLResponseByHTTPPost(Request, out samlResponseXml, out relayState);

            // Check if SAMLMessage has a valid signature
            if (!SAMLMessageSignature.Verify(samlResponseXml))
                throw new Exception("SAMLResponse has invalid signature");

            // Check if signature used to sign the SAMLResponse is the correct one
            if (SAMLMetadataSignature.GetCertificate(IdPMetadata.DocumentElement).Thumbprint != SAMLMessageSignature.GetCertificate(samlResponseXml).Thumbprint)
                throw new Exception("SAMLResponse signed with an invalid certificate");

            // Read the attributes from the SAMLResponse
            SAMLResponse samlResponse = new SAMLResponse(samlResponseXml);
            ReadAttributes(samlResponse.GetAssertions());

            // Do your own login magic
            LoginAtYourSite(m_uid);

            // Relaystate contains original redirectUrl
            RelayState cachedRelayState = RelayStateCache.Remove(relayState);
            string redirectUrl = cachedRelayState.ResourceURL;
            
            // Do the redirect
            if (!string.IsNullOrEmpty(redirectUrl))
                Response.Redirect(redirectUrl + "?uid=" + m_uid);
        }

        private void ReadAttributes(IList<SAMLAssertion> samlAssertions)
        {
            SAMLAssertion samlAssertion = samlAssertions.First();
            m_uid = samlAssertion.GetAttributeValue("uid");
            string m_firstName = samlAssertion.GetAttributeValue("givenName");
            string m_tussenvoegsel = samlAssertion.GetAttributeValue("nlEduPersonTussenvoegsels");
            string m_lastName = samlAssertion.GetAttributeValue("sn");
            string m_role = samlAssertion.GetAttributeValue("eduPersonAffiliation");
            string m_brin = samlAssertion.GetAttributeValue("nlEduPersonHomeOrganizationId");
            string m_school = samlAssertion.GetAttributeValue("nlEduPersonHomeOrganization");
        }

        private void LoginAtYourSite(string user_id)
        {
            // Perform local login...
        }
    }
}