﻿using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Xml;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;

using ComponentSpace.SAML2;
using ComponentSpace.SAML2.Assertions;
using ComponentSpace.SAML2.Bindings;
using ComponentSpace.SAML2.Metadata;
using ComponentSpace.SAML2.Protocols;
using ComponentSpace.SAML2.Profiles.SSOBrowser;

namespace SAML2SPExample
{
    public partial class Login : System.Web.UI.Page
    { 
        X509Certificate2 m_certificate;           
        private string m_EntityID;

        private string m_ssoUrl;

        protected void Page_Load(object sender, EventArgs e)
        {
            // Import self-signed certificate
            m_certificate = new X509Certificate2("c:/myfiles/SAMLCertificate.p12", "hello", X509KeyStorageFlags.Exportable);            

            // Define EntityID
            m_EntityID = "http://www.SPExample.org";

            // Read metadata staging
            XmlDocument IdPMetadata = new XmlDocument();
            IdPMetadata.PreserveWhitespace = true;

            using (XmlReader xmlreader = XmlReader.Create("https://hub-s.entree.kennisnet.nl/openaselect/profiles/saml2/"))
                IdPMetadata.Load(xmlreader); 

            // Verify Metadata signature
            if (!SAMLMetadataSignature.Verify(IdPMetadata.DocumentElement))
                throw new Exception("Kennisnet metadata has invalid signature");

            // Get IDPSSODescriptor and SSO Url from metadata
            ReadMetadata(IdPMetadata.DocumentElement);

            // Create relaystate
            string redirectUrl = "default.aspx";
            string relayState = RelayStateCache.Add(new RelayState(redirectUrl, null));

            // Create the AuthnRequest
            XmlElement authnRequestXml = CreateAuthnRequest();

            // Redirect to the IdP
            ServiceProvider.SendAuthnRequestByHTTPPost(Response, m_ssoUrl, authnRequestXml, relayState);
            Response.End();
        }

        // Reads IdP metadata
        private void ReadMetadata(XmlElement xmlElement)
        {
            if (EntityDescriptor.IsValid(xmlElement))
            {
                EntityDescriptor entityDescriptor = new EntityDescriptor(xmlElement); 
                IDPSSODescriptor idpSSODescriptor = entityDescriptor.IDPSSODescriptors.FirstOrDefault();

                m_ssoUrl = (from endpointType in idpSSODescriptor.SingleSignOnServices
                            where endpointType.Binding == "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
                            select endpointType.Location).FirstOrDefault();

                if (string.IsNullOrEmpty(m_ssoUrl))
                    throw new ArgumentException("No url specified for specified binding");
            }
            else
            {
                throw new ArgumentException("Expecting entity descriptor");
            }
        }  

        // Create the AuthnRequest
        private XmlElement CreateAuthnRequest()
        {
            string assertionConsumerServiceURL = CreateAbsoluteURL("AssertionConsumerService.aspx");
            AuthnRequest authnRequest = new AuthnRequest();
            authnRequest.Destination = m_ssoUrl;
            authnRequest.Issuer = new Issuer(m_EntityID);
            authnRequest.ForceAuthn = false;
            authnRequest.NameIDPolicy = new NameIDPolicy(null, null, true);
            authnRequest.ProtocolBinding = SAMLIdentifiers.BindingURIs.HTTPPost;
            authnRequest.AssertionConsumerServiceURL = assertionConsumerServiceURL;
            XmlElement authnRequestXml = authnRequest.ToXml();

            // Add signature
            SAMLMessageSignature.Generate(authnRequestXml, m_certificate.PrivateKey, m_certificate);
            return authnRequestXml;
        }

        // Create an absolute URL from an application relative URL
        private string CreateAbsoluteURL(string relativeURL)
        {
            return new Uri(Request.Url, ResolveUrl(relativeURL)).ToString();
        }
    }
}